with Gwindows.Gstrings;
with Interfaces.C;
with Ada.Text_Io; use Ada;

package body Gwindows.Combo_Boxes.Ex_Combo_Boxes is

   CB_ADDSTRING               : constant := 323;
   CB_GETITEMDATA             : constant := 16#150#;
   CB_SETITEMDATA             : constant := 16#151#;
   CB_SETITEMHEIGHT           : constant := 16#0153#;
   CB_FINDSTRING              : constant := 16#014C#;
   CB_FINDSTRINGEXACT         : constant := 344;
   CB_ERR                     : constant := -1;


   function Find (Combo      : in ex_drop_down_combo_box_type;
                  Value      : in GString;
                  Start_Item : in Natural        := 0)
                 return Natural
   is
      C_Value : GString_C := GWindows.GStrings.To_GString_C (Value);

      function SendMessage
        (hwnd   : Interfaces.C.long := Handle (Combo);
         uMsg   : Interfaces.C.int  := CB_FINDSTRING;
         wParam : Interfaces.C.long := Interfaces.C.Long(Start_Item - 1);
         lParam : GString_C         := C_Value)
        return Integer;
      pragma Import (StdCall, SendMessage,
                       "SendMessage" & Character_Mode_Identifier);
   begin
      return SendMessage + 1;
   end Find;
   -------------------------------------------------------------------------------
   function Find_Exact (Combo      : in ex_drop_down_combo_box_type;
                  Value      : in GString;
                  Start_Item : in Natural        := 0)
                 return Natural
   is
      C_Value : GString_C := GWindows.GStrings.To_GString_C (Value);

      function SendMessage
        (hwnd   : Interfaces.C.long := Handle (Combo);
         uMsg   : Interfaces.C.int  := CB_FINDSTRINGEXACT;
         wParam : Interfaces.C.long := Interfaces.C.Long(Start_Item - 1);
         lParam : GString_C         := C_Value)
        return Integer;
      pragma Import (StdCall, SendMessage,
                       "SendMessage" & Character_Mode_Identifier);
   begin
      return SendMessage + 1;
   end Find_Exact;
   -------------------------------------------------------------------------------
   procedure Add(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                 Value: in Gstring;
                 Index: out Natural) is
      C_Value : GString_C := GWindows.GStrings.To_GString_C (Value);

      function SendMessage(hwnd   : Interfaces.C.long := Handle (Combo);
                           uMsg   : Interfaces.C.int  := CB_ADDSTRING;
                           wParam : Interfaces.C.long := 0;
                           lParam : GString_C         := C_Value)
      return Interfaces.C.Int;
      pragma Import (StdCall, SendMessage,
                       "SendMessage" & Character_Mode_Identifier);
   begin
      -- + 1 is necessary to be compatible to Davids framework
      Index := Integer(Sendmessage) + 1;
   end Add;
   ------------------------------------------------------------------------------
   procedure Set_Item_Id(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                         Item: in natural;
                         Id: in Integer)is

      procedure SendMessage(hwnd   : Interfaces.C.long := Handle (Combo);
                            uMsg   : Interfaces.C.int  := CB_SETITEMDATA;
                            wParam : Interfaces.C.long := Interfaces.C.Long(Item - 1);
                            lParam : Interfaces.C.long := Interfaces.C.Long(Id));
      pragma Import (StdCall, SendMessage,
                       "SendMessage" & Character_Mode_Identifier);
   begin
      sendmessage;
   end Set_Item_Id;
   ------------------------------------------------------------------------------
   function Get_Item_Id(Combo: in Ex_Drop_Down_Combo_Box_Type;
                        Item: in natural) return Integer is

      function SendMessage(hwnd   : Interfaces.C.long := Handle (Combo);
                           uMsg   : Interfaces.C.int  := CB_GETITEMDATA;
                           wParam : Interfaces.C.long := Interfaces.C.Long(Item - 1);
                           lParam : Interfaces.C.long := 0)
                          return Interfaces.C.Long;
      pragma Import (StdCall, SendMessage,
                       "SendMessage" & Character_Mode_Identifier);
   begin
      return Integer(Sendmessage);
   end Get_Item_Id;
   ------------------------------------------------------------------------------
   procedure On_Edit_Change (Combo : in out Ex_Drop_Down_Combo_Box_Type) is
      Index: natural;
      Len: Natural;
   begin
      if Combo.Autocomplete then
         Len := Text(Combo)'Last;
         Index := Find(Combo, Text(Combo));
         if Len > Combo.Last_Text_Len then  -- new char was added by the user
            if Index /= 0 then -- entry found
               Current(Combo, Index);
               Set_Edit_Selection(Combo, Len, Text(Combo)'Last);
               Combo.Last_Text_Len := Len;  -- remember length
            else  -- entry not found
               if Combo.Force_Valid_Text then -- recover last valid entry
                  if Len = 1 then
                     Text(Combo, "");
                  else
                     Index := Find(Combo, Text(Combo)(1..Len-1));
                     Current(Combo, Index);
                     Set_Edit_Selection(Combo, Len - 1, Text(Combo)'Last);
                  end if;
               end if;
            end if;
         else
            Combo.Last_Text_Len := Len;  -- remember length
         end if;
      end if;
      On_Edit_Change(Drop_Down_Combo_box_Type(Combo));
   end On_Edit_Change;
   ------------------------------------------------------------------------------
   procedure Set_Autocomplete(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                              Autocomplete: in Boolean) is
   begin
      Combo.Autocomplete := Autocomplete;
   end Set_Autocomplete;
   ------------------------------------------------------------------------------
   procedure Force_Valid_Text(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                              Force: in Boolean) is
   begin
      Combo.Force_Valid_Text := Force;
   end Force_Valid_Text;
   ------------------------------------------------------------------------------
   procedure Set_Selection_Field_Height(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                                        Height: in Natural)is
      use Interfaces.C;
      procedure SendMessage(hwnd   : Interfaces.C.long := Handle (Combo);
                            uMsg   : Interfaces.C.int  := CB_SETITEMHEIGHT;
                            wParam : Interfaces.C.long := -1;
                            lParam : Interfaces.C.long := Interfaces.C.Long(Height));
      pragma Import (StdCall, SendMessage,
                       "SendMessage" & Character_Mode_Identifier);

   begin
      Sendmessage;
   end Set_Selection_Field_Height;
end Gwindows.Combo_Boxes.Ex_Combo_Boxes;
