------------------------------------------------------------------------------
--                                                                          --
--                                                                          --
--                     Gwindows.Common_controls.Ex_lv                       --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                            $Revision: 1.18 $                             --
--                                                                          --
--                            Konad GmbH 2004                               --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. It is distributed in the hope that it will be useful,  but WITHOUT --
-- ANY WARRANTY;  without  even the  implied warranty of MERCHANTABILITY or --
-- FITNESS FOR A PARTICULAR PURPOSE.    See the GNU General  Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- More information about GWINDOWS and the most current public version can  --
-- be located on the web at http://www.gnavi.org/gwindows                   --
--                                                                          --
------------------------------------------------------------------------------
-- This Package provides an Extension of the List_View_Control_TYpe
-- (David Botton). Actually the following features are implemented
--
-- -> Coloring on the subitem - level
-- -> Several extended styles (see below) are supported
-- -> items may be displayed with icons
-- -> Column sorting is supported for one or two columns
-- -> Column headers may be colored (text and background)
-- -> Column headers may be given an image
-- -> Columns may be permuted at once
------------------------------------------------------------------------------

with Gwindows.Base; use Gwindows.Base;
with Gwindows.Colors; use Gwindows.Colors;
with Gwindows.Image_Lists.Ex_Image_lists; use Gwindows.Image_Lists.Ex_Image_lists;
with Ada.Unchecked_Conversion; use Ada;
with System.Address_To_Access_conversions;
with Gwindows.Types; use Gwindows.Types;
with Interfaces.C; use Interfaces;

package Gwindows.Common_Controls.Ex_LV is

   type Ex_List_View_Control_Type is new List_View_Control_type with private;
   type Ex_List_View_Control_Access is access all Ex_List_View_Control_Type;
   type Pointer_To_Ex_List_View_Control_Class is private;

   procedure CreateEx(Control    : in out Ex_List_View_Control_Type;
                      Parent     : in out Gwindows.Base.Base_Window_Type'Class;
                      Left       : in     Integer;
                      Top        : in     Integer;
                      Width      : in     Integer;
                      Height     : in     Integer;
                      Label_Editing : in  Boolean                              := False;
                      NocolumnHeader  : in Boolean                            := False;
                      Color_Mode : in     Boolean                              := True;
                      Selection  : in     List_View_Control_Select_Type        := Single;
                      View       : in     List_View_Control_View_Type          := List_View;
                      Sort       : in     List_View_Control_Sort_Type          := No_Sorting;
                      Arrange    : in     Boolean                              := True;
                      Align      : in     List_View_Control_Alignment_Type     := Align_Left;
                      Show       : in     Boolean                              := True;
                      Is_Dynamic : in     Boolean                              := False       );
   -- creates an instance of ex_list_view_control_type
   --
   -- with parameter color_mode you may control the internal
   -- color-administration
   -- false -> no coloring of items or subitems possible
   -- true -> items and subitems may be colored individually
   -- items/subitems are given via parameters
   --
   -- Nocolumnheader=true means that the list_view does not
   -- contain a header.
   --
   -- Label_Editing=true means that you may edit item (not subitem!)
   -- text in place, but the parent window has to process the LVN_ENDLABELEDIT
   -- notification message

   type List_View_Extended_Style_Type is (Grid, Flatsb, Headerdragdrop, Fullrowselect);
   -- all styles may be combined

   procedure Set_Extended_Style (Control : in     Ex_List_View_Control_Type;
                                 Style   : in     List_View_Extended_Style_Type );
   -- sets an extended style

   procedure Remove_Extended_Style (Control : in     Ex_List_View_Control_Type;
                                    Style   : in     List_View_Extended_Style_Type );
   -- removes an extended style

   procedure Clear_Extended_Styles (Control : in     Ex_List_View_Control_Type );
   -- removes all extended styles

   procedure Insert_Item (Control : in out Ex_List_View_Control_Type;
                          Text    : in     Gstring;
                          Index   : in     Integer;
                          Icon    : in     Integer                   := 0 );
   -- inserts an item
   -- the parameter icon refers to an icon's index in the controls imagelist
   -- see below (set_image_list)

   procedure Delete_Item (Control : in out Ex_List_View_Control_Type;
                          Index   : in     Integer);
   -- remove an item

   procedure Delete_column (Control : in out Ex_List_View_Control_Type;
                            column   : in     Integer);
   -- remove a whole column

   procedure Set_Sub_Item (Control   : in out Ex_List_View_Control_Type;
                           Text      : in     Gstring;
                           Index     : in     Integer;
                           Sub_Index : in     Integer                    );
   -- sets a subitem

   procedure Set_Image_List (Control    : in     Ex_List_View_Control_Type;
                             Image_List : in     Ex_Image_List_Type            );
   -- sets the controls imagelist

   procedure Set_Header_Image_List (Control    : in     Ex_List_View_Control_Type;
                                    Image_List : in     Ex_Image_List_Type            );
   -- set header image list (for e.g. the little arrows when sorted)

   --------------------------------------------------------------------------------
   --                         S O R T I N G
   --------------------------------------------------------------------------------
   type Sort_Direction_Type is(Up, Down, auto);

   procedure Init_Date_format(Format: in String);
   -- describes how a columns content is interpreted as a date
   --
   -- DD - Day
   -- MM - Month
   -- YYYY - Year
   -- hh   - hours (24)
   -- mi   - minutes
   -- ss   - seconds
   --
   -- The year *must* be given with four digits or be omitted
   --
   -- with Init_Date_Format() you may change the date format to adjust to your needs
   -- The default date format is set to: DD.MM.YYYY hh:mi


   procedure Sort_Column (Control : in out Ex_List_View_Control_Type;
                          Column  : in     Integer;
                          Sort : in Sort_Direction_Type := Auto;
                          Is_Date : in Boolean := False;
                          Up_Sort_Icon : in Integer := 0;
                          Down_Sort_Icon : in Integer := 1);
   -- sorts with respect to a column
   -- sort = up -> sort ascending
   -- sort = down -> sort descending
   -- sort = auto -> sorts with the known "toggle behaviour":
   --  first call of sort_column sorts ascending
   --  further calls on the SAME column reverse order
   --
   -- If Is_Date=true then the columns content is interpreted as
   -- a date according to Date_Format and the list is sorted
   -- chronologically

   subtype Limited_Sort_Direction_Type is Sort_Direction_Type range Up..Down;

   procedure Sort_Two_Column (Control : in out Ex_List_View_Control_Type;
                              First_Column  : in     Integer;
                              First_Sort : in Limited_Sort_Direction_Type := up;
                              First_Is_Date: in Boolean := False;
                              Second_Column  : in     Integer;
                              second_Sort : in Limited_Sort_Direction_Type := up;
                              second_Is_Date: in Boolean := False;
                              First_Up_Sort_Icon : in Integer := 0;
                              First_Down_Sort_Icon : in Integer := 1;
                              second_Up_Sort_Icon : in Integer := 2;
                              second_Down_Sort_Icon : in Integer := 3);
   -- this procedure sorts the list lexicographically with respect to
   -- First_Column and Second_Column. The sort icons are taken from
   -- the list headers image list if it is set otherwise from the
   -- controls image list.

   procedure Set_Text_Color (Control : in     Ex_List_View_Control_Type;
                             Color   : in     Color_Type                 );
   -- sets textcolor for the whole control
   -- no effect when Color_mode = true!

   procedure Set_TextBk_Color (Control : in   out  Ex_List_View_Control_Type;
                               Color   : in     Color_Type                 );
   -- sets background-textcolor for the whole control
   -- no effect when Color_mode = true!

   procedure Set_Header_Color(Control : in out Ex_List_View_Control_Type;
                              Column  : in     Integer;
                              Text_Color: in Color_Type;
                              Back_Color: in Color_type);
   -- sets text and/or background color for the header

   procedure Set_Back_Color (Control : in  out  Ex_List_View_Control_Type;
                             Color   : in     Color_Type                 );
   -- sets the background color for the area without rows

   procedure Set_Item_Color (Control    : in out Ex_List_View_Control_Type;
                             Text_Color : in     Color_Type;
                             Bk_Color   : in     Color_Type;
                             Index      : in     Integer;
                             Sub_Index  : in     Integer                   );
   -- sets the color individually for an item/subitem depending on parameter
   -- color_mode in createEx

   procedure Set_Alternate_Back_Item_Color (Control    : in out Ex_List_View_Control_Type;
                                            Bk_Color   : in     Color_Type;
                                            Index      : in     Integer;
                                            Sub_Index  : in     Integer                   );
   -- sets the alternate back color individually for an item/subitem depending on parameter

   procedure Use_Alternate_Coloring(Control    : in out Ex_List_View_Control_Type;
                                    Coloring:  in Boolean := True);
   -- enable or disable the alternate coloring

   procedure Column_Autosize(Control: in Ex_List_View_Control_Type;
                             Column : in Integer);
   -- automatically adjust column width according to content

   procedure Insert_Column (Control : in out Ex_List_View_Control_Type;
                            Text    : in     GString;
                            Index   : in     Integer;
                            Width   : in     Integer);
   -- inserts a column

   type Int_Array_Type is array(Positive range <>) of Integer;
   type Int_Array_Access is access all Int_Array_Type;

   procedure Set_Columnorder(Control: in Ex_List_View_Control_Type;
                             Columnorder: in Int_Array_Type);
   -- sets the order of columns according to the permutation stored
   -- in array Columnorder: Columnorder'First=1, Columnorder'Last=
   -- number of columns, Columnorder(I) = new Position of Column I

   procedure Get_Columnorder(Control: in Ex_List_View_Control_Type;
                             Columnorder: in out Int_Array_Type);
   -- after a call to Get_Columnorder(), Columnorder(I) yields the
   -- actual position of Column I.

   function Get_Column_Count(Control: in Ex_List_View_Control_Type) return Integer;
   -- returns number of columns

   function Get_Column_Text(Control: in Ex_List_View_Control_Type;
                            Column: in Integer) return Gstring;
   -- the caption of the column - that is the column header text

   function Get_Column_Width(Control: in Ex_List_View_Control_Type;
                             Column: in Integer) return Integer;
   -- returns the column width in pixels

   procedure Item_At_Position (Control  : in     Ex_List_View_Control_Type;
                               Position : in     Gwindows.Types.Point_Type;
                               Item     : in out Integer;
                               Subitem  : in out Integer);
   -- necessary overwrite because of an error in Davids function:
   -- in gwindows-common_controls.adb, line 1981, the point of
   -- the lvhittestinfo-structure is always set to (0,0).

   function Get_Subitem_Rect(Control  : in Ex_List_View_Control_Type;
                             Item     : in Integer;
                             Subitem  : in Integer) return Gwindows.Types.Rectangle_Type;
   -- returns the rectangle occupied by one cell (subitem)

   procedure Set_Header_image(Control : in out Ex_List_View_Control_Type;
                              Column: in Integer;
                              Image_Index: in integer);
   -- sets an image in a columns header. If the header does not have an image
   -- list for its own the image is taken from the controls image list. If the
   -- control does not have one, nothing happens (no error).

   procedure Clear_Header_Image (Control : in out Ex_List_View_Control_Type;
                                 Column  : in     Integer);
   -- removes an image from a columns header

   procedure Get_Approximate_View (List   : in     Ex_List_View_Control_Type;
                                   Items  : in     Integer                   := -1;
                                   Width  :    out Integer;
                                   Height :    out integer);

   -- returns the approximate width and height in pixels needed to display the given number
   -- of items.
   -- Items = -1 (default) means: all items

   type Direction_Type is (Hor, Ver);

   procedure Get_Scroll_Info(List: in Ex_List_View_Control_Type;
                             Scroll_Direction: in Direction_Type;
                             Scroll_Min: out Integer;
                             Scroll_Max: out Integer;
                             Scroll_Pos: out Integer;
                             Scroll_Page: out Integer;
                             Scroll_Track_pos: out Integer);

   -- You may call Get_Scroll_Info() to get
   -- [Scroll_Min, Scroll_Max] the interval in which scrolling is possible
   -- Scroll_Pos = Position of the left upper pixel which is visible in
   -- the scroll window.
   -- Scroll_Page = size of the window under which scrolling is performed
   --               (Hor -> width, Ver -> height)
   -- Scroll_Track_Pos = Like Scroll_Pos but suitable if Get_Scroll_Info
   --                    is called while SB_THUMBTRACK is processed


   ---------------------
   -- event-handling --
   ---------------------

   --------------------------------------------------------------------------------
   --                         Header events
   --------------------------------------------------------------------------------
   type Header_Click_Event is access procedure
     (Window    : in out Ex_List_View_control_Type;
      Column   : in Integer);

   procedure On_Header_Click (Control : in out Ex_List_View_Control_Type;
                              Column  : in     Integer                    );

   procedure On_Header_Click_Handler (Control : in out Ex_List_View_Control_Type;
                                      Handler : in     Header_Click_Event         );

   procedure Fire_On_Header_Click (Control : in out Ex_List_View_Control_Type;
                                   Column  : in     Integer                    );

   ---------------------------------------------------------------------------------
   --                               Item Events
   ---------------------------------------------------------------------------------
   type Item_Changed_Event is access procedure
     (Window    : in out Ex_List_View_control_Type;
      index   : in Integer);

   procedure On_Item_changed (Control : in out Ex_List_View_Control_Type;
                              index  : in     Integer                    );

   procedure On_Item_changed_Handler (Control : in out Ex_List_View_Control_Type;
                                      Handler : in     Item_changed_Event         );

   procedure Fire_On_Item_changed (Control : in out Ex_List_View_Control_Type;
                                   index  : in     Integer                    );

   --------------------------------------------------------------------------------
   --                       General Events
   --------------------------------------------------------------------------------
   procedure On_Create (Control : in out Ex_List_View_Control_Type );

   procedure On_Notify (Window       : in out Ex_List_View_Control_Type;
                        Message      : in     Gwindows.Base.Pointer_To_Notification;
                        Control      : in     Gwindows.Base.Pointer_To_Base_Window_Class;
                        Return_Value : in out Interfaces.C.Long                           );
   procedure On_Message(Window       : in out Ex_List_View_Control_Type;
                        message      : in     Interfaces.C.unsigned;
                        wParam       : in     Interfaces.C.int;
                        lParam       : in     Interfaces.C.int;
                        Return_Value : in out Interfaces.C.long);

   procedure On_Destroy (Window : in out Ex_List_View_Control_Type);


   ----------------------------------------------------------------------------------
private

    type Scrollinfo_Type is
      record
         CbSize: C.unsigned;
         FMask: C.Unsigned;
         NMin: C.Int;
         NMax: C.Int;
         NPage: C.unsigned;
         NPos: C.Int;
         NTrackPos: C.Int;
      end record;


    type Scrollinfo_Access is access all Scrollinfo_Type;

   default_Date_Format: String := "DD.MM.YYYY hh:mi";

   type Date_Format_Element_Type is
      record
         first: Natural := 0;
         last : Natural := 0;
      end record;

   type Date_Format_Array_Type is array(1..6) of Date_Format_Element_Type;

   type Date_Format_Type is
      record
         Arr: Date_Format_Array_Type;
         Is_Valid: Boolean := False;
      end record;

   Date_Convert: Date_Format_Type;

   type Extended_Data_Type is
      record
         ID                    : Integer      := -1;
         Text_Color            : Color_Type   := To_Color(0, 0, 0);
         Back_Color            : Color_Type   := To_Color(255, 255, 255);
         Alternate_Back_Color  : Color_Type   := To_Color(255, 255, 255);
      end record;

   type Extended_Data_Array_Type is array(natural range <>) of Extended_Data_Type;

   package Address_Conversion is new
     System.Address_To_Access_Conversions(Extended_Data_Array_type);

   subtype Extended_Data_Array_Access is Address_Conversion.Object_Pointer;

   type Header_Item_Color_Type is
      record
         Text_Color: Color_Type := To_Color(0,0,0);
         back_Color: Color_Type := System_Color(Color_Scrollbar);
      end record;

   package Header_Item_Address_Conversion is new
      System.Address_To_Access_Conversions(Header_Item_Color_Type);

      subtype Header_Item_Color_Access is
        Header_Item_Address_Conversion.Object_Pointer;

   -- to remember the last sort operation
   type Sort_Type is
      record
         Two_Column_Sorting: Boolean := False;
         First_Sort_Column: Integer := -1;
         First_Column_Sort_Direction: Sort_Direction_Type := Up;
         second_Sort_Column: Integer := -1;
         Second_Column_Sort_Direction: Sort_Direction_Type := Up;
      end record;

   type Ex_List_View_Control_Type is new List_View_Control_Type with
      record
         On_Header_Click_Event : Header_Click_Event      := null;
         On_Item_Changed_Event : Item_Changed_Event      := null;
         Color_Mode            : Boolean                 := true;
         Sorting               : Sort_Type;
         Column_Count          : Integer                 := 0;
         Use_Alternate_Coloring: Boolean                 := False;
      end record;

   type Pointer_To_Ex_List_View_Control_Class is access all Ex_List_View_Control_Type'Class;

end Gwindows.Common_Controls.Ex_LV;
