with Gwindows.Base; use Gwindows.Base;
with Interfaces.C;
with Interfaces.C.strings; use Interfaces.C.strings;
with Gwindows.Gstrings; use Gwindows.Gstrings;

package body Gwindows.Common_Controls.Ex_Tb is

   CCS_NORESIZE        :constant := 4;
   WM_USER             :constant := 16#400#;
   TB_ADDBUTTONS       :constant := Wm_User + 20;
   Tbstate_Enabled     :constant := 16#4#;
   TBSTYLE_FLAT        :constant := 16#800#;
   TBSTYLE_LIST        :constant := 16#1000#;
   BTNS_CHECK          :constant := 2;
   TB_BUTTONSTRUCTSIZE :constant  := (WM_USER + 30);
   BTNS_AUTOSIZE       :constant :=  16#10#;
   I_IMAGENONE         :constant :=    (-2);
   TBSTYLE_TOOLTIPS    :constant :=  16#100#;
   TB_AUTOSIZE         :constant :=  (WM_USER + 33);

   type Tbbutton is
      record
         Image   : Integer                    := 0;
         Command : Integer                    := 0;
         State   : Interfaces.C.Unsigned_Char := 0;
         Style   : Interfaces.C.Unsigned_Char := 0;
         Pad1    : Interfaces.C.Unsigned_Char := 0;
         pad2    : Interfaces.C.Unsigned_Char := 0;
         Data    : Integer                    := 0;
         Istring : Chars_Ptr;
      end record;


   --------------
   -- CreateEx --
   --------------

   procedure CreateEx
     (Control       : in out Ex_Toolbar_Control_Type;
      Parent        : in out GWindows.Base.Base_Window_Type'Class;
      Left          : in     Integer;
      Top           : in     Integer;
      Width         : in     Integer;
      Height        : in     Integer;
      Max_Buttons   : in     positive                             := 100;
      Text_Position : in     Text_Position_Type                   := right;
      Is_Flat       : in     Boolean                              := True;
      Show          : in     Boolean                              := True;
      Is_Dynamic    : in     Boolean                              := False) is

      use Interfaces.C;

      Styles : Interfaces.C.Unsigned := CCS_NORESIZE + TBSTYLE_TOOLTIPS;

      procedure Sendmessage_buttonstructsize(Hwnd   : Interfaces.C.Long := Handle (Control);
                                             Umsg   : Interfaces.C.Int  := TB_BUTTONSTRUCTSIZE;
                                             Wparam : Integer           := 20;
                                             Lparam : Integer           := 0                );
      pragma Import (Stdcall, Sendmessage_buttonstructsize,
                       "SendMessage" & Character_Mode_Identifier);

   begin
      if Text_Position = Right then
         Styles := Styles + TBSTYLE_LIST;
      end if;
      if IS_Flat then
         Styles := Styles + TBSTYLE_Flat;
      end if;

      Gwindows.Base.Create_Control (Base_Window_Type(Control),
                                    Parent,
                                    "ToolbarWindow32",
                                    "",
                                    Left, Top, Width, Height,
                                    0, Styles,
                                    Is_Dynamic => Is_Dynamic);

      Sendmessage_buttonstructsize;

      Control.Tooltip_Array := new Tooltipdata_Array_Type(1..Max_Buttons);

      if Show then
         Gwindows.Common_controls.Show (Toolbar_Control_Type(Control));
      end if;

   end CreateEx;

   --------------------
   -- Set_Image_list --
   --------------------

   procedure Set_Image_List(Control : in out Ex_Toolbar_Control_Type;
                            List    : in     Ex_Image_List_Type)is
      use Gwindows.Image_Lists;
   begin
      Set_Image_List(Toolbar_Control_Type(Control), Image_List_Type(List));
   end Set_Image_List;

   ----------------
   -- Add_Button --
   ----------------

   procedure Add_Button (Control     : in out Ex_Toolbar_Control_Type;
                         Image_Index : in     Natural;
                         Command     : in     Integer;
                         Button_Style: in Button_Style_Type;
                         Text        : in GString := " ";
                         Tooltip     : in GString := " ";
                         None_Image: in Boolean := false) is

        type Button_Array is array (1 .. 1) of Tbbutton;
        Tb : Button_Array;

        procedure Sendmessage_addbutton (
                                         Hwnd   : Interfaces.C.Long := Handle (Control);
                                         Umsg   : Interfaces.C.Int  := Tb_Addbuttons;
                                         Wparam : Integer           := 1;
                                         Lparam : Button_Array      := Tb                );
        pragma Import (Stdcall, Sendmessage_addbutton,
                         "SendMessage" & Character_Mode_Identifier);
        use Interfaces.C;
   begin
      case Button_Style is
         when Check_Style =>
            Tb (1).Style := BTNS_CHECK;
         when others =>
            Tb (1).Style := 0;
      end case;

      Tb (1).Style := Tb(1).Style + BTNS_AUTOSIZE;
      Tb (1).IString := New_String(To_String(Text));
      Tb (1).Image := Image_Index;
      Tb (1).Command := Command;
      Tb (1).State := Tbstate_Enabled;

      if None_Image then
         Tb (1).Image := I_IMAGENONE;
      end if;

      if Control.Next_Tooltip > 100 then
         return;
      end if;

      Control.Tooltip_Array(Control.Next_Tooltip).Command := Command;
      Control.Tooltip_Array(Control.Next_Tooltip).tooltip := new GString'(Tooltip);
      Control.Next_Tooltip := Control.Next_Tooltip + 1;

      Sendmessage_addbutton;
   end Add_Button;

   -----------------
   -- Get_Tooltip --
   -----------------

   function Get_Tooltip(Control: in Ex_Toolbar_Control_Type;
                        command: in Integer) return Gstring is
   begin
      for I in 1..Control.Next_Tooltip - 1 loop
         if Control.Tooltip_Array(I).Command = command then
            return Control.Tooltip_Array(I).Tooltip.all;
         end if;
      end loop;
      return "";

   exception
      when others =>
         return "";
   end Get_Tooltip;

end Gwindows.Common_Controls.Ex_Tb;
