with Ada.Text_Io; use Ada.Text_Io;

package body Gwindows.Edit_Boxes.Ex_Multi_Line_Edit_Boxes is

   WM_KEYDOWN                 : constant := 256;
   WM_KEYUP                   : constant := 257;

     procedure Translate_Key(wParam      : in     Interfaces.C.int;
                             Special_Key :    out Gwindows.Windows.Special_Key_Type;
                             Key         :    out GCharacter);

   ---------------------
   -- On_Character_Up --
   ---------------------

   procedure On_Character_Up
     (Window      : in out ex_multi_line_edit_box_type;
      Special_Key : in     Gwindows.Windows.Special_Key_Type;
      Value       : in     GCharacter)
   is
   begin
      Fire_On_Character_Up (Window, Special_Key, Value);
   end On_Character_Up;

   -----------------------
   -- On_Character_Down --
   -----------------------

   procedure On_Character_Down
     (Window      : in out ex_multi_line_edit_box_type;
      Special_Key : in     Gwindows.Windows.Special_Key_Type;
      Value       : in     GCharacter)
   is
   begin
      Fire_On_Character_Down (Window, Special_Key, Value);
   end On_Character_Down;

   -----------------------------
   -- On_Character_Up_Handler --
   -----------------------------

   procedure On_Character_Up_Handler
     (Window  : in out ex_multi_line_edit_box_type;
      Handler : in     Character_Event)
   is
   begin
      Window.On_Character_Up_Event := Handler;
   end On_Character_Up_Handler;

   -------------------------------
   -- On_Character_Down_Handler --
   -------------------------------

   procedure On_Character_Down_Handler
     (Window  : in out ex_multi_line_edit_box_type;
      Handler : in     Character_Event)
   is
   begin
      Window.On_Character_Down_Event := Handler;
   end On_Character_Down_Handler;

   --------------------------
   -- Fire_On_Character_Up --
   --------------------------

   procedure Fire_On_Character_Up
     (Window      : in out ex_multi_line_edit_box_type;
      Special_Key : in     Gwindows.Windows.Special_Key_Type;
      Value       : in     GCharacter)
   is
   begin
      if Window.On_Character_Up_Event /= null then
         Window.On_Character_Up_Event(Window, Special_Key, Value);
      end if;
   end Fire_On_Character_Up;

   ----------------------------
   -- Fire_On_Character_Down --
   ----------------------------

   procedure Fire_On_Character_Down
     (Window      : in out ex_multi_line_edit_box_type;
      Special_Key : in     Gwindows.Windows.Special_Key_Type;
      Value       : in     GCharacter)
   is
   begin
      if Window.On_Character_Down_Event /= null then
         Window.On_Character_Down_Event(Window, Special_Key, Value);
      end if;
   end Fire_On_Character_Down;

   procedure On_Message (Window       : in out ex_multi_line_edit_box_type;
                         message      : in     Interfaces.C.unsigned;
                         wParam       : in     Interfaces.C.int;
                         lParam       : in     Interfaces.C.int;
                         Return_Value : in out Interfaces.C.long)is
   begin
      case Message is
         when WM_KEYDOWN =>
            declare
               Special : Gwindows.Windows.Special_Key_Type;
               Key     : GCharacter;
            begin
               Translate_Key (wParam, Special, Key);
               On_Character_Down (Window, Special, Key);
               Return_Value := 0;
            end;

         when WM_KEYUP =>
            declare
               Special : Gwindows.Windows.Special_Key_Type;
               Key     : GCharacter;
            begin
               Translate_Key (wParam, Special, Key);
               On_Character_Up (Window, Special, Key);
               Return_Value := 0;
            end;
         when others =>
            On_Message(Multi_Line_Edit_Box_Type(Window),
                       Message, Wparam, Lparam, Return_Value);
      end case;
   end On_Message;

   -------------------
   -- Translate_Key --
   -------------------

   procedure Translate_Key(wParam      : in     Interfaces.C.int;
                           Special_Key :    out Gwindows.Windows.Special_Key_Type;
                           Key         :    out GCharacter)
   is
      use Interfaces.C;
      use Gwindows.Windows;
      type State_Array is array (0 .. 255) of Character;
      Keyboard_State : State_Array := (others => ' ');

      procedure GetKeyboardState
        (lpbKeyState : State_Array := Keyboard_State);
      pragma Import (StdCall, GetKeyboardState, "GetKeyboardState");

      Key_Value : aliased Interfaces.C.unsigned := 0;

      procedure ToAscii
        (uVirtKey    : in     Interfaces.C.int      := wParam;
         uScanCode   : in     Interfaces.C.int      := 0;
         lpbKeyState : in     State_Array           := Keyboard_State;
         lpChar      : access Interfaces.C.unsigned := Key_Value'Access;
         uFlags      : in     Interfaces.C.int      := 0);
      pragma Import (StdCall, ToAscii, "ToAscii");

      procedure ToUnicode
        (uVirtKey    : in     Interfaces.C.int      := wParam;
         uScanCode   : in     Interfaces.C.int      := 0;
         lpbKeyState : in     State_Array           := Keyboard_State;
         lpChar      : access Interfaces.C.unsigned := Key_Value'Access;
         ccBuff      : in     Integer               := 1;
         uFlags      : in     Interfaces.C.int      := 0);
      pragma Import (StdCall, ToUnicode, "ToUnicode");

      VK_ESCAPE                  : constant := 27;
      VK_PAUSE                   : constant := 19;
      VK_CAPITAL                 : constant := 20;
      VK_PRIOR                   : constant := 33;
      VK_NEXT                    : constant := 34;
      VK_END                     : constant := 35;
      VK_HOME                    : constant := 36;
      VK_LEFT                    : constant := 37;
      VK_UP                      : constant := 38;
      VK_RIGHT                   : constant := 39;
      VK_DOWN                    : constant := 40;
      VK_SNAPSHOT                : constant := 44;
      VK_INSERT                  : constant := 45;
      VK_DELETE                  : constant := 46;
      VK_F1                      : constant := 112;
      VK_F2                      : constant := 113;
      VK_F3                      : constant := 114;
      VK_F4                      : constant := 115;
      VK_F5                      : constant := 116;
      VK_F6                      : constant := 117;
      VK_F7                      : constant := 118;
      VK_F8                      : constant := 119;
      VK_F9                      : constant := 120;
      VK_F10                     : constant := 121;
      VK_F11                     : constant := 122;
      VK_F12                     : constant := 123;
      VK_NUMLOCK                 : constant := 144;
      VK_SCROLL                  : constant := 145;
   begin
      Key := GCharacter'Val (0);
      case wParam is
         when VK_ESCAPE =>
            Special_Key := Escape;
         when VK_PAUSE =>
            Special_Key := Pause;
         when VK_CAPITAL =>
            Special_Key := Caps_Lock;
         when VK_PRIOR =>
            Special_Key := Page_Up;
         when VK_NEXT =>
            Special_Key := Page_Down;
         when VK_END =>
            Special_Key := End_Key;
         when VK_HOME =>
            Special_Key := Home_Key;
         when VK_LEFT =>
            Special_Key := Left_Key;
         when VK_UP =>
            Special_Key := Up_Key;
         when VK_RIGHT =>
            Special_Key := Right_Key;
         when VK_DOWN =>
            Special_Key := Down_Key;
         when VK_SNAPSHOT =>
            Special_Key := Print_Screen;
         when VK_INSERT =>
            Special_Key := Insert;
         when VK_DELETE =>
            Special_Key := Delete;
         when VK_F1 =>
            Special_Key := F1;
         when VK_F2 =>
            Special_Key := F2;
         when VK_F3 =>
            Special_Key := F3;
         when VK_F4 =>
            Special_Key := F4;
         when VK_F5 =>
            Special_Key := F5;
         when VK_F6 =>
            Special_Key := F6;
         when VK_F7 =>
            Special_Key := F7;
         when VK_F8 =>
            Special_Key := F8;
         when VK_F9 =>
            Special_Key := F9;
         when VK_F10 =>
            Special_Key := F10;
         when VK_F11 =>
            Special_Key := F11;
         when VK_F12 =>
            Special_Key := F12;
         when VK_NUMLOCK =>
            Special_Key := Number_Lock;
         when VK_SCROLL =>
            Special_Key := Scroll_Lock;
         when others =>
            Special_Key := None;
            GetKeyboardState;
            if Character_Mode = Unicode then
               ToUnicode;
            else
               ToAscii;
            end if;

            if Key_Value > 255 then
               Key_Value := Key_Value - (Key_Value and 16#FF00#);
            end if;
            Key := GCharacter'Val (Key_Value);
      end case;

   end Translate_Key;

end Gwindows.Edit_Boxes.Ex_Multi_Line_Edit_Boxes;
