with Gwindows.Gstrings;
with GWindows.Internal;

package body Gwindows.Image_Lists.Ex_Image_Lists is

   function Get_Color_Flag(Color_Depth: in Color_Depth_Type) return Natural;

   ILC_COLOR4              : constant := 4;
   ILC_COLOR8              : Constant := 8;
   ILC_COLOR16             : constant := 16#10#;
   ILC_COLOR24             : constant := 16#18#;
   ILC_COLOR32             : constant := 16#20#;

   --------------
   -- CreateEx --
   --------------

   procedure CreateEx (List          : in out Ex_Image_List_Type;
                       Width, Height : in     Positive;
                       Initial_Size  : in     Positive;
                       Grow_By       : in     Natural         := 1;
                       Color_Depth  : in Color_Depth_Type    := Depth_32)
   is
      function ImageList_Create
        (CX    : Positive := Width;
         XY    : Positive := Height;
         FLAGS : Natural  := 0;
         Init  : Positive := Initial_Size;
         Grow  : Natural  := Grow_By)
        return GWindows.Types.Handle;
      pragma Import (StdCall, ImageList_Create, "ImageList_Create");
   begin
      Handle (List, ImageList_Create(FLAGS => Get_Color_Flag(Color_Depth)));
   end CreateEx;


   procedure CreateEx (List    : in out Ex_Image_List_Type;
                       Name    : in     GString;
                       Width   : in     Positive;
                       Grow_By : in     Natural         := 1;
                       Color_Depth  : in Color_Depth_Type    := Depth_32)

   is
      CLR_NONE   : constant := 16#FFFFFFFF#;
      CLR_DEFAULT  : constant := 16#FF000000#;

      C_Text : Interfaces.C.char_array :=
        Interfaces.C.To_C (GWindows.GStrings.To_String (Name));

      function ImageList_LoadImage
        (HINST : Interfaces.C.long      := GWindows.Internal.Current_hInstance;
         Name  : Interfaces.C.char_array := C_Text;
         CX    : Positive                := Width;
         Grow  : Natural                 := Grow_By;
         CREF  : Interfaces.C.unsigned   := CLR_NONE;
         UT    : Integer                 := 0;
         FLAGS : Natural                 := 0)
        return GWindows.Types.Handle;
      pragma Import (StdCall, ImageList_LoadImage, "ImageList_LoadImage");
   begin
      Handle (List, ImageList_LoadImage(FLAGS => Get_Color_Flag(Color_Depth)));
   end CreateEx;

   --------------------
   -- body functions --
   --------------------

   --------------------
   -- Get_Color_Flag --
   --------------------

   function Get_Color_Flag(Color_Depth: in Color_Depth_Type) return natural is
   begin
      case Color_Depth is
         when Depth_4 =>
            return ILC_COLOR4;
         when Depth_8 =>
            return ILC_COLOR8;
         when Depth_16 =>
            return ILC_COLOR16;
         when Depth_24 =>
            return ILC_COLOR24;
         when Depth_32 =>
            return ILC_COLOR32;
         when others =>
            return 0;
      end case;
   end Get_Color_Flag;

   -----------------------------------------------------------------------------

   procedure Create_From_File (List      : in out Ex_Image_List_Type;
                               File_Name : in     GString;
                               Width     : in     Positive;
                               Grow_By   : in     Natural         := 1)
   is
      pragma Warnings (Off, List);

      C_Text : Interfaces.C.char_array :=
        Interfaces.C.To_C (GWindows.GStrings.To_String (File_Name));
      LR_LOADFROMFILE : constant := 16;

      function ImageList_LoadImage
        (HINST : Interfaces.C.long     := GWindows.Internal.Current_hInstance;
         Name  : Interfaces.C.char_array := C_Text;
         CX    : Positive              := Width;
         Grow  : Natural               := Grow_By;
         CREF  : Interfaces.C.unsigned := 16#FF000000#;
         UT    : Integer               := 0;
         FLAGS : Natural               := LR_LOADFROMFILE)
        return GWindows.Types.Handle;
      pragma Import (StdCall, ImageList_LoadImage, "ImageList_LoadImage");
   begin
      Handle (List, ImageList_LoadImage);
   end Create_From_File;

end Gwindows.Image_Lists.Ex_Image_Lists;
